@@ -9,7 +9,20 @@
 import enum
 
 
-__all__ = ['DataGeneratorResults', 'DataGeneratorVariableResults', 'OutputResults', 'ReportFormat']
+__all__ = ['VariableResults', 'DataGeneratorResults', 'DataSetResults', 'ReportResults', 'ReportFormat']
+
+
+class VariableResults(dict):
+    """ Dictionary that maps the ids of variables (:obj:`Variable`) to their results (:obj:`numpy.ndarray`)
+
+    * Keys (:obj:`str`): ids of variables (:obj:`Variable`)
+    * Values (:obj:`numpy.ndarray`): result of each variable
+
+        * Steady-state tasks of non-spatial models: results should be arrays of shape ``(1,)``
+        * One-step tasks of non-spatial models: results should be arrays of shape ``(2,)``
+        * Uniform time course tasks of non-spatial models: results should be arrays of shape ``(number_of_points + 1,)``
+    """
+    pass
 
 
 class DataGeneratorResults(dict):
@@ -25,34 +38,24 @@ class DataGeneratorResults(dict):
     pass
 
 
-class DataGeneratorVariableResults(dict):
-    """ Dictionary that maps the ids of variables (:obj:`DataGeneratorVariable`) to their results (:obj:`numpy.ndarray`)
+class DataSetResults(dict):
+    """ Dictionary that maps the ids of data sets to their results (:obj:`numpy.ndarray`)
 
-    * Keys (:obj:`str`): ids of variables (:obj:`DataGeneratorVariable`)
-    * Values (:obj:`numpy.ndarray`): result of each variable
+    * Keys (:obj:`str`): ids of data sets
+    * Values (:obj:`numpy.ndarray`): result of each data set
 
-        * Steady-state tasks of non-spatial models: results should be arrays of shape ``(1,)``
-        * One-step tasks of non-spatial models: results should be arrays of shape ``(2,)``
-        * Uniform time course tasks of non-spatial models: results should be arrays of shape ``(number_of_points + 1,)``
+        * Steady-state tasks of non-spatial models: results should be arrays of shape ``(number of data sets, 1)``
+        * One-step tasks of non-spatial models: results should be arrays of shape ``(number of data sets, 2)``
+        * Uniform time course tasks of non-spatial models: results should be arrays of shape ``(number_of_points + 1)``
     """
     pass
 
 
-class OutputResults(dict):
-    """ Dictionary that maps the ids of outputs (e.g., :obj:`Report`) to their results (:obj:`pandas.DataFrame`)
+class ReportResults(dict):
+    """ Dictionary that maps the ids of reports (e.g., :obj:`Report`) to their results (:obj:`DataSetResults`)
 
-    * Keys (:obj:`str`): ids of outputs (e.g., :obj:`Report`)
-    * Values (:obj:`pandas.DataFrame`): result of each output
-
-        * Data:
-
-            * Steady-state tasks of non-spatial models: results should be arrays of shape ``(number of data sets, 1)``
-            * One-step tasks of non-spatial models: results should be arrays of shape ``(number of data sets, 2)``
-            * Uniform time course tasks of non-spatial models: results should be arrays of shape ``(number of data sets, number_of_points + 1)``
-
-        * Indices (row labels)
-
-            * Reports: equal to the ids of the data sets if each report
+    * Keys (:obj:`str`): ids of reports (e.g., :obj:`Report`)
+    * Values (:obj:`DataSetResults`): result of each report
     """
     pass
 
@@ -63,3 +66,5 @@ class ReportFormat(str, enum.Enum):
     h5 = 'h5'
     hdf = 'h5'
     hdf5 = 'h5'
+    tsv = 'tsv'
+    xlsx = 'xlsx'
