@@ -17,7 +17,7 @@ import numpy  # noqa: F401
 
 __all__ = [
     'ModelLanguage',
-    'DataGeneratorVariableSymbol',
+    'Symbol',
     'SedDocument',
     'Simulation',
     'SteadyStateSimulation',
@@ -28,11 +28,15 @@ __all__ = [
     'Model',
     'ModelChange',
     'ModelAttributeChange',
+    'AddElementModelChange',
+    'ReplaceElementModelChange',
+    'RemoveElementModelChange',
+    'ComputeModelChange',
     'AbstractTask',
     'Task',
     'DataGenerator',
-    'DataGeneratorVariable',
-    'DataGeneratorParameter',
+    'Variable',
+    'Parameter',
     'Output',
     'Report',
     'DataSet',
@@ -41,19 +45,21 @@ __all__ = [
     'AxisScale',
     'Curve',
     'Surface',
+    'MATHEMATICAL_FUNCTIONS',
+    'RESERVED_MATHEMATICAL_SYMBOLS',
 ]
 
 
 class ModelLanguage(str, enum.Enum):
     """ Model language """
-    BGNL = 'urn:sedml:language:bgnl'
+    BNGL = 'urn:sedml:language:bngl'
     CellML = 'urn:sedml:language:cellml'
     NeuroML = 'urn:sedml:language:neuroml'
     SBML = 'urn:sedml:language:sbml'
     VCML = 'urn:sedml:language:vcml'
 
 
-class DataGeneratorVariableSymbol(str, enum.Enum):
+class Symbol(str, enum.Enum):
     """ Variable sumbol """
     time = 'urn:sedml:symbol:time'
 
@@ -427,17 +433,14 @@ class ModelChange(abc.ABC):
     """ A change to a model
 
     Attributes:
-        name (:obj:`str`): name
         target (:obj:`str`): path to the model element that should be changed
     """
 
-    def __init__(self, name=None, target=None):
+    def __init__(self, target=None):
         """
         Args:
-            name (:obj:`str`, optional): name
             target (:obj:`str`, optional): path to the model element that should be changed
         """
-        self.name = name
         self.target = target
 
     @abc.abstractmethod
@@ -459,7 +462,6 @@ class ModelChange(abc.ABC):
             :obj:`bool`: :obj:`True`, if two model changes are equal
         """
         return self.__class__ == other.__class__ \
-            and self.name == other.name \
             and self.target == other.target
 
 
@@ -501,6 +503,147 @@ class ModelAttributeChange(ModelChange):
             and self.new_value == other.new_value
 
 
+class AddElementModelChange(ModelChange):
+    """ An addition of an element to a model
+
+    Attributes:
+        target (:obj:`str`): path to the parent of the new element(s)
+        new_elements (:obj:`str`): new element(s)
+    """
+
+    def __init__(self, target=None, new_elements=None):
+        """
+        Args:
+            target (:obj:`str`, optional): path to the parent of the new element(s)
+            new_elements (:obj:`str`): new element(s)
+        """
+        super(AddElementModelChange, self).__init__(target=target)
+        self.new_elements = new_elements
+
+    def to_tuple(self):
+        """ Get a tuple representation
+
+        Returns:
+            :obj:`tuple` of :obj:`str`: tuple representation
+        """
+        return (self.target, self.new_elements)
+
+    def is_equal(self, other):
+        """ Determine if model attribute changes are equal
+
+        Args:
+            other (:obj:`AddElementModelChange`): another content item
+
+        Returns:
+            :obj:`bool`: :obj:`True`, if two model attribute changes are equal
+        """
+        return super(AddElementModelChange, self).is_equal(other) \
+            and self.new_elements == other.new_elements
+
+
+class ReplaceElementModelChange(ModelChange):
+    """ A replacement of an element of a model
+
+    Attributes:
+        target (:obj:`str`): path to the element to replace
+        new_elements (:obj:`str`): new element(s)
+    """
+
+    def __init__(self, target=None, new_elements=None):
+        """
+        Args:
+            target (:obj:`str`, optional): path to the element to replace
+            new_elements (:obj:`str`): new element(s)
+        """
+        super(ReplaceElementModelChange, self).__init__(target=target)
+        self.new_elements = new_elements
+
+    def to_tuple(self):
+        """ Get a tuple representation
+
+        Returns:
+            :obj:`tuple` of :obj:`str`: tuple representation
+        """
+        return (self.target, self.new_elements)
+
+    def is_equal(self, other):
+        """ Determine if model attribute changes are equal
+
+        Args:
+            other (:obj:`ReplaceElementModelChange`): another content item
+
+        Returns:
+            :obj:`bool`: :obj:`True`, if two model attribute changes are equal
+        """
+        return super(ReplaceElementModelChange, self).is_equal(other) \
+            and self.new_elements == other.new_elements
+
+
+class RemoveElementModelChange(ModelChange):
+    """ A removal of an element from a model
+
+    Attributes:
+        target (:obj:`str`): path to the element to remove
+    """
+
+    def to_tuple(self):
+        """ Get a tuple representation
+
+        Returns:
+            :obj:`tuple` of :obj:`str`: tuple representation
+        """
+        return (self.target,)
+
+
+class ComputeModelChange(ModelChange):
+    """ A replacement of an element of a model
+
+    Attributes:
+        target (:obj:`str`): path to the element to replace
+        variables (:obj:`list` of :obj:`Variable`): variables
+        parameters (:obj:`list` of :obj:`Parameter`): parameters
+        math (:obj:`str`): mathematical expression
+    """
+
+    def __init__(self, target=None, variables=None, parameters=None, math=None):
+        """
+        Args:
+            target (:obj:`str`, optional): path to the element to replace
+            variables (:obj:`list` of :obj:`Variable`, optional): variables
+            parameters (:obj:`list` of :obj:`Parameter`, optional): parameters
+            math (:obj:`str`, optional): mathematical expression
+        """
+        super(ComputeModelChange, self).__init__(target=target)
+        self.variables = variables or []
+        self.parameters = parameters or []
+        self.math = math
+
+    def to_tuple(self):
+        """ Get a tuple representation
+
+        Returns:
+            :obj:`tuple` of :obj:`str`: tuple representation
+        """
+        return (self.target,
+                tuple(none_sorted(variable.to_tuple() for variable in self.variables)),
+                tuple(none_sorted(parameter.to_tuple() for parameter in self.parameters)),
+                self.math)
+
+    def is_equal(self, other):
+        """ Determine if model attribute changes are equal
+
+        Args:
+            other (:obj:`ComputeModelChange`): another content item
+
+        Returns:
+            :obj:`bool`: :obj:`True`, if two model attribute changes are equal
+        """
+        return super(ComputeModelChange, self).is_equal(other) \
+            and are_lists_equal(self.variables, other.variables) \
+            and are_lists_equal(self.parameters, other.parameters) \
+            and self.math == other.math
+
+
 class AbstractTask(abc.ABC):
     """ Base class for tasks
 
@@ -570,8 +713,8 @@ class Task(AbstractTask):
             :obj:`tuple` of :obj:`str`: tuple representation
         """
         return (self.id, self.name,
-                self.model.to_tuple() if self.model else None,
-                self.simulation.to_tuple() if self.simulation else None)
+                self.model.id if self.model else None,
+                self.simulation.id if self.simulation else None)
 
     def is_equal(self, other):
         """ Determine if tasks are equal
@@ -584,9 +727,9 @@ class Task(AbstractTask):
         """
         return super(Task, self).is_equal(other) \
             and ((self.model is None and self.model == other.model)
-                 or (self.model is not None and self.model.is_equal(other.model))) \
+                 or (self.model is not None and other.model is not None and self.model.id == other.model.id)) \
             and ((self.simulation is None and self.simulation == other.simulation)
-                 or (self.simulation is not None and self.simulation.is_equal(other.simulation)))
+                 or (self.simulation is not None and other.simulation is not None and self.simulation.id == other.simulation.id))
 
 
 class DataGenerator(object):
@@ -595,8 +738,8 @@ class DataGenerator(object):
     Attributes:
         id (:obj:`str`): id
         name (:obj:`str`): name
-        variables (:obj:`list` of :obj:`DataGeneratorVariable`): variables
-        parameters (:obj:`list` of :obj:`DataGeneratorParameters`): variables
+        variables (:obj:`list` of :obj:`Variable`): variables
+        parameters (:obj:`list` of :obj:`Parameter`): variables
         math (:obj:`str`): mathematical expression
     """
 
@@ -605,8 +748,8 @@ class DataGenerator(object):
         Args:
             id (:obj:`str`, optional): id
             name (:obj:`str`, optional): name
-            variables (:obj:`list` of :obj:`DataGeneratorVariable`, optional): variables
-            parameters (:obj:`list` of :obj:`DataGeneratorParameters`, optional): variables
+            variables (:obj:`list` of :obj:`Variable`, optional): variables
+            parameters (:obj:`list` of :obj:`Parameter`, optional): variables
             math (:obj:`str`, optional): mathematical expression
         """
         self.id = id
@@ -644,8 +787,8 @@ class DataGenerator(object):
             and self.math == other.math
 
 
-class DataGeneratorVariable(object):
-    """ A model variable involved in a data generator
+class Variable(object):
+    """ A model variable
 
     Attributes:
         id (:obj:`str`): id
@@ -680,14 +823,14 @@ class DataGeneratorVariable(object):
             :obj:`tuple` of :obj:`str`: tuple representation
         """
         return (self.id, self.name, self.target, self.symbol,
-                self.task.to_tuple() if self.task else None,
-                self.model.to_tuple() if self.model else None)
+                self.task.id if self.task else None,
+                self.model.id if self.model else None)
 
     def is_equal(self, other):
         """ Determine if data generator variables are equal
 
         Args:
-            other (:obj:`DataGeneratorVariable`): another content item
+            other (:obj:`Variable`): another content item
 
         Returns:
             :obj:`bool`: :obj:`True`, if two data generator variables are equal
@@ -698,12 +841,12 @@ class DataGeneratorVariable(object):
             and self.target == other.target \
             and self.symbol == other.symbol \
             and ((self.task is None and self.task == other.task)
-                 or (self.task is not None and self.task.is_equal(other.task))) \
+                 or (self.task is not None and other.task is not None and self.task.id == other.task.id)) \
             and ((self.model is None and self.model == other.model)
-                 or (self.model is not None and self.model.is_equal(other.model)))
+                 or (self.model is not None and other.model is not None and self.model.id == other.model.id))
 
 
-class DataGeneratorParameter(object):
+class Parameter(object):
     """ A parameter involved in a data generator
 
     Attributes:
@@ -735,7 +878,7 @@ class DataGeneratorParameter(object):
         """ Determine if data generator parameters are equal
 
         Args:
-            other (:obj:`DataGeneratorParameter`): another content item
+            other (:obj:`Parameter`): another content item
 
         Returns:
             :obj:`bool`: :obj:`True`, if two data generator parameters are equal
@@ -746,42 +889,6 @@ class DataGeneratorParameter(object):
             and self.value == other.value
 
 
-MATHEMATICAL_FUNCTIONS = {
-    'root': lambda x, n: x**(1 / float(n)),
-    'abs': abs,
-    'exp': math.exp,
-    'ln': math.log,
-    'log': math.log,
-    'floor': math.floor,
-    'ceiling': math.ceil,
-    'factorial': math.factorial,
-    'sin': math.sin,
-    'cos': math.cos,
-    'tan': math.tan,
-    'sec': mpmath.sec,
-    'csc': mpmath.csc,
-    'cot': mpmath.cot,
-    'sinh': math.sinh,
-    'cosh': math.cosh,
-    'tanh': math.tanh,
-    'sech': mpmath.sech,
-    'csch': mpmath.csch,
-    'coth': mpmath.coth,
-    'arcsin': math.asin,
-    'arccos': math.acos,
-    'arctan': math.atan,
-    'arcsec': mpmath.asec,
-    'arccsc': mpmath.acsc,
-    'arccot': mpmath.acot,
-    'arcsinh': math.asinh,
-    'arccosh': math.acosh,
-    'arctanh': math.atanh,
-    'arcsech': mpmath.asech,
-    'arccsch': mpmath.acsch,
-    'arccoth': mpmath.acoth,
-}
-
-
 class Output(abc.ABC):
     """ An output
 
@@ -893,7 +1000,7 @@ class DataSet(object):
             :obj:`tuple` of :obj:`str`: tuple representation
         """
         return (self.id, self.name, self.label,
-                self.data_generator.to_tuple() if self.data_generator is not None else None)
+                self.data_generator.id if self.data_generator is not None else None)
 
     def is_equal(self, other):
         """ Determine if data sets are equal
@@ -909,8 +1016,9 @@ class DataSet(object):
             and self.name == other.name \
             and self.label == other.label \
             and ((self.data_generator is None and self.data_generator == other.data_generator)
-                 or (self.data_generator is not None and self.data_generator.is_equal(other.data_generator))) \
-
+                 or (self.data_generator is not None
+                     and other.data_generator is not None
+                     and self.data_generator.id == other.data_generator.id))
 
 
 class Plot2D(Output):
@@ -1045,8 +1153,8 @@ class Curve(object):
         return (self.id, self.name,
                 self.x_scale.value if self.x_scale else None,
                 self.y_scale.value if self.y_scale else None,
-                self.x_data_generator.to_tuple() if self.x_data_generator else None,
-                self.y_data_generator.to_tuple() if self.y_data_generator else None)
+                self.x_data_generator.id if self.x_data_generator else None,
+                self.y_data_generator.id if self.y_data_generator else None)
 
     def is_equal(self, other):
         """ Determine if curves are equal
@@ -1063,9 +1171,13 @@ class Curve(object):
             and self.x_scale == other.x_scale \
             and self.y_scale == other.y_scale \
             and ((self.x_data_generator is None and self.x_data_generator == other.x_data_generator)
-                 or (self.x_data_generator is not None and self.x_data_generator.is_equal(other.x_data_generator))) \
+                 or (self.x_data_generator is not None
+                     and other.x_data_generator is not None
+                     and self.x_data_generator.id == other.x_data_generator.id)) \
             and ((self.y_data_generator is None and self.y_data_generator == other.y_data_generator)
-                 or (self.y_data_generator is not None and self.y_data_generator.is_equal(other.y_data_generator)))
+                 or (self.y_data_generator is not None
+                     and other.y_data_generator is not None
+                     and self.y_data_generator.id == other.y_data_generator.id))
 
 
 class Surface(object):
@@ -1115,9 +1227,9 @@ class Surface(object):
                 self.x_scale.value if self.x_scale else None,
                 self.y_scale.value if self.y_scale else None,
                 self.z_scale.value if self.z_scale else None,
-                self.x_data_generator.to_tuple() if self.x_data_generator else None,
-                self.y_data_generator.to_tuple() if self.y_data_generator else None,
-                self.z_data_generator.to_tuple() if self.z_data_generator else None)
+                self.x_data_generator.id if self.x_data_generator else None,
+                self.y_data_generator.id if self.y_data_generator else None,
+                self.z_data_generator.id if self.z_data_generator else None)
 
     def is_equal(self, other):
         """ Determine if surfaces are equal
@@ -1135,8 +1247,59 @@ class Surface(object):
             and self.y_scale == other.y_scale \
             and self.z_scale == other.z_scale \
             and ((self.x_data_generator is None and self.x_data_generator == other.x_data_generator)
-                 or (self.x_data_generator is not None and self.x_data_generator.is_equal(other.x_data_generator))) \
+                 or (self.x_data_generator is not None
+                     and other.x_data_generator is not None
+                     and self.x_data_generator.id == other.x_data_generator.id)) \
             and ((self.y_data_generator is None and self.y_data_generator == other.y_data_generator)
-                 or (self.y_data_generator is not None and self.y_data_generator.is_equal(other.y_data_generator))) \
+                 or (self.y_data_generator is not None
+                     and other.y_data_generator is not None
+                     and self.y_data_generator.id == other.y_data_generator.id)) \
             and ((self.z_data_generator is None and self.z_data_generator == other.z_data_generator)
-                 or (self.z_data_generator is not None and self.z_data_generator.is_equal(other.z_data_generator)))
+                 or (self.z_data_generator is not None
+                     and other.z_data_generator is not None
+                     and self.z_data_generator.id == other.z_data_generator.id))
+
+
+MATHEMATICAL_FUNCTIONS = {
+    'root': lambda x, n: x**(1 / float(n)),
+    'abs': abs,
+    'exp': math.exp,
+    'ln': math.log,
+    'log': math.log,
+    'floor': math.floor,
+    'ceiling': math.ceil,
+    'factorial': math.factorial,
+    'sin': math.sin,
+    'cos': math.cos,
+    'tan': math.tan,
+    'sec': mpmath.sec,
+    'csc': mpmath.csc,
+    'cot': mpmath.cot,
+    'sinh': math.sinh,
+    'cosh': math.cosh,
+    'tanh': math.tanh,
+    'sech': mpmath.sech,
+    'csch': mpmath.csch,
+    'coth': mpmath.coth,
+    'arcsin': math.asin,
+    'arccos': math.acos,
+    'arctan': math.atan,
+    'arcsec': mpmath.asec,
+    'arccsc': mpmath.acsc,
+    'arccot': mpmath.acot,
+    'arcsinh': math.asinh,
+    'arccosh': math.acosh,
+    'arctanh': math.atanh,
+    'arcsech': mpmath.asech,
+    'arccsch': mpmath.acsch,
+    'arccoth': mpmath.acoth,
+}
+
+RESERVED_MATHEMATICAL_SYMBOLS = {
+    'true': True,
+    'false': False,
+    'notanumber': math.nan,
+    'pi': math.pi,
+    'infinity': math.inf,
+    'exponentiale': math.e,
+}
