@@ -10,6 +10,7 @@ from .data_model import EnvironmentVariable  # noqa: F401
 import cement
 import platform
 import sys
+import termcolor
 import types  # noqa: F401
 
 __all__ = [
@@ -25,7 +26,7 @@ def build_cli(cli_name=None, cli_version=None,
     The command-line application will have two inputs
 
     * A path to a COMBINE/OMEX archive that describes one or more simulations or one or more models
-    * A path to a directory to store the ouputs of the execution of the simulations defined in the archive
+    * A path to a directory to store the outputs of the execution of the simulations defined in the archive
 
     The command-line application will also support two additional commands
 
@@ -71,7 +72,7 @@ def build_cli(cli_name=None, cli_version=None,
         for env_var in sorted(environment_variables, key=lambda var: var.name):
             description_value += "\n  '{}': {}".format(env_var.name, env_var.description)
             if env_var.options:
-                option_values = sorted("'" + val + "'" for val in env_var.options.__members__.values())
+                option_values = ["'" + val + "'" for val in env_var.options]
                 description_value += "\n    Options: {}".format(', '.join(option_values))
             if env_var.default:
                 description_value += "\n    Default value: '{}'".format(env_var.default)
@@ -131,7 +132,7 @@ def build_cli(cli_name=None, cli_version=None,
             try:
                 combine_archive_executer(args.archive, args.out_dir)
             except Exception as exception:
-                raise SystemExit(str(exception)) from exception
+                raise SystemExit(termcolor.colored(str(exception), 'red')) from exception
 
     class App(cement.App):
         """ Command line application """
