@@ -6,7 +6,7 @@
 :License: MIT
 """
 
-from .data_model import EnvironmentVariable, AlgorithmSubstitutionPolicy
+from .data_model import EnvironmentVariable, AlgorithmSubstitutionPolicy, ALGORITHM_SUBSTITUTION_POLICY_LEVELS
 
 __all__ = [
     'ENVIRONMENT_VARIABLES',
@@ -16,8 +16,8 @@ ENVIRONMENT_VARIABLES = {
     AlgorithmSubstitutionPolicy: EnvironmentVariable(
         name='ALGORITHM_SUBSTITUTION_POLICY',
         description='Policy for substituting alternative algorithms.',
-        options=AlgorithmSubstitutionPolicy,
-        default=AlgorithmSubstitutionPolicy.SAME_FRAMEWORK,
+        options=[policy.value for policy, _ in sorted(ALGORITHM_SUBSTITUTION_POLICY_LEVELS.items(), key=lambda policy_level: policy_level[1])],
+        default=AlgorithmSubstitutionPolicy.SIMILAR_VARIABLES,
         more_info_url='https://biosimulators.org/standards/simulator-interfaces',
     )
 }
